/*
 * linux/saudit.h
 *
 * Original:
 * Copyright (c) 1999-2004 InterSect Alliance Pty Ltd 
 *                         - http://www.intersectalliance.com/
 * Additions:
 * Copyright (c) 2004 Silicon Graphics, Inc. All rights reserved.
 */

#ifndef __C2_AUDIT_H
#define __C2_AUDIT_H

#include <linux/compiler.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/mm.h>
#include <net/sock.h>

// If the user has not turned on C2 Auditing, define all 'public' routines out.

#if defined(CONFIG_C2_AUDIT)

// I was going to use __NR_xxx, but we need audit events for events that
// may NOT be system calls specifically (eg: login, connect/accept)
#define AUDIT_open		1
#define AUDIT_mkdir		2
#define AUDIT_unlink		3
#define AUDIT_rmdir		4
#define AUDIT_chown		5
#define AUDIT_chmod		6
#define AUDIT_symlink		7
#define AUDIT_link		8
#define AUDIT_rename		9
#define AUDIT_mknod		10
#define AUDIT_truncate		11
#define AUDIT_ftruncate		12
#define AUDIT_chroot		13
#define AUDIT_execve		14
#define AUDIT_exit		15
#define AUDIT_setuid		16
#define AUDIT_setreuid		17
#define AUDIT_setresuid		18
#define AUDIT_setgid		19
#define AUDIT_setregid		20
#define AUDIT_setresgid		21
#define AUDIT_create_module	22
#define AUDIT_delete_module	23
#define AUDIT_reboot		24
#define AUDIT_connect		25
#define AUDIT_accept		26
#define AUDIT_mount		27
#define AUDIT_umount		28
#define AUDIT_fork		29

// Size of the bitmask array that we need to store the information
// associated with whether an audit event is currently turned on.
#define MAXAUDIT			AUDIT_fork

// ioctl modes - note that '1' doesnt seem to work. Added 10.
#define AUDIT_STOP			10  // stop auditing
#define AUDIT_START			11  // start auditing
#define AUDIT_INFO			12  // Give me a list of events
					    // currently active, and other
					    // info such as the process ID
#define AUDIT_LOSTEVENTS		13
#define AUDIT_FLUSH			14  // Stop all events.
#define AUDIT_EVENT_ON			15  // Turn on a selected event
#define AUDIT_EVENT_OFF			16  // Turn off a selected event
#define AUDIT_DELIVERY 			17  // Do we want to guarantee audit event delivery
#define AUDIT_TOTALEVENTS		18  // How many events have been received this session?

#define AUDIT_HIGHWATERMARK_MEM		19  // Set the High Water Mark memory usage
#define AUDIT_HIGHWATERMARK_PER		20  // Set the High Water Mark memory as a percentage mem.
#define AUDIT_HIGHWATERMARK_PAUSE	21  // Set the High Water Mark pause percetage 
#define AUDIT_LOWWATERMARK_PAUSE        22  // Set the Low Water Mark to resume
#define AUDIT_HIGHWATERMARK_NICE        23  // Set the High Water Mark to change nice value
#define AUDIT_LOWWATERMARK_NICE         24  // Set the Low Water Mark to restore nice value
#define AUDIT_HIGHWATER_NICE_VAL	25  // The nice value to change the audit daemon to





#define AUDIT_CLASS_NONE	0
#define AUDIT_CLASS_IO		1	// Input/output (file opens)
#define AUDIT_CLASS_PC		2	// Process Control
#define AUDIT_CLASS_EXEC	3	// Execution
#define AUDIT_CLASS_NET		4	// Network related
#define AUDIT_CLASS_ADMIN	5	// Administrative events
#define AUDIT_CLASS_CH		6	// CHMOD event. Might not use this for anything else.
#define AUDIT_CLASS_CP		7	// Where more than one pathname is required
#define AUDIT_CLASS_SU		8	// SetUID
#define AUDIT_CLASS_AD		9	// Admin such as create/delete module

#define SNAREAUDIT_MAJOR_VERSION 0
#define SNAREAUDIT_MINOR_VERSION 9
#define SNAREAUDIT_PATCH_VERSION 7


// /proc entry
#define AUDITDEV_NAME  "snare"			// device name in /dev and /proc/devices
#define AUDITINFO_NAME	"snareinfo"		// Information about the process
#define AUDITDEV_FILE  "/proc/snare"		// full file name
#define AUDITINFO_FILE "/proc/snareinfo"	// Information about the process.
#define MAX_PATH   512				// NOTE: will migrgate this to PATH_MAX eventually
#define MAXCOMMAND 25


// This contains the details that are common to ALL audit events.
typedef struct
{
	unsigned short event_class;		// event class. Each class has a predictable format for tokens.
	unsigned short event_id;		// number of the event
	unsigned short event_size;		// size of the event struct - don't include header
						// since it's always the same
	struct timeval time;			// time

	int user_id;				// User ID
	int euser_id;				// Effective User ID
	int group_id;				// Group ID
	int egroup_id;				// Effective Group ID

	int returncode;				// Make sure that this is big enough to contain the largest returncode.

	pid_t pid;				// process ID.
	pid_t ppid;				// Parent process ID.
	char processname[MAXCOMMAND];	// Same as in /usr/include/linux/sched.h for current->comm
} header_token;

typedef struct
{
	char path[MAX_PATH];
} path_token;

typedef struct
{
	int mode;			// How the file was attempted to be opened or created
	unsigned long	createmode;	// Flags associated with the file creation. Ulong for mknod.
} attributes_token;

typedef struct
{
	int owner;			// new owner of a file - was uid_t, but these are different between kernel and user.
	int group;			// new group of a file
} owner_token;

typedef struct
{
	char args[MAX_PATH];		// Should really allocate more here. Whats is the max command line size?
} execargs_token;

// System calls like setuid
typedef struct
{
	int id;		// uid/gid/euid depending on the call
	int rid;	// ruid/rgid
	int sid;	// suid/sgid
} target_token;

// Network connections
typedef struct
{
	char src_ip[40];	// String containing source dotted ip address - 40 bytes, for IPv6
	int src_port;		// Source port
	char dst_ip[40];	// String containing destination dotted ip address - 40 bytes, for IPv6
	int dst_port;		// Destination port
	int protocol;		// Protocol type - IPPROTO_UDP or IPPROTO_TCP
} connection_token;



// Now for the audit event classes

// Just a bare class with the minimal data
// note that this will mean that EVERY class must start with header_token
typedef struct
{
	header_token	t_header;
} null_class;
	

// NOTE: ANY CLASS STRUCTURE SHOULD HAVE THE RETURN TOKEN AS THE SECOND ELEMENT.
// SEE AUDITD FOR MORE INFO.
// io - reads/writes
typedef struct
{
	header_token		t_header;
	path_token		t_path;
	path_token		t_pwd;	// Working directory
	attributes_token	t_attributes;
} io_class;

typedef struct
{
	header_token		t_header;
	path_token		t_path;
	path_token		t_pwd;	// Working directory
	owner_token		t_owner;
} ch_class;

typedef struct
{
	header_token		t_header;
	path_token		t_path;
	path_token		t_pwd;	// Working directory
	execargs_token		t_execargs;
	// environment variables too?
} ex_class;

typedef struct
{
	header_token		t_header;
} pc_class;

// copy one file to another (amongst others - eg: symlink)
typedef struct
{
	header_token		t_header;
	path_token		t_sourcepath;
	path_token		t_pwd;	// Working directory
	path_token		t_destpath;
} cp_class;

typedef struct
{
	header_token		t_header;
	target_token		t_target;	// target UID or GID.. I really only need a single value here.
} su_class;

typedef struct
{
	header_token		t_header;
	connection_token	t_connection;	
} nt_class;	// Network

typedef struct
{
	header_token		t_header;
	path_token		t_name;		// Name of the module loaded / removed
} ad_class;	// General Administrative

struct _auditnode
{
	void * location;	// Location in RAM of the allocated chunk
	int size;		// Size of the chunk
	struct _auditnode *next;	// Next node in the series.
};

typedef struct _auditnode AuditNode;


#ifdef __KERNEL__
#include <linux/sched.h>

int _audit_mknod(const char * kfile, const char __user * ufile, int mode, dev_t dev, int retval);
int _audit_execve(const char * kfilename, const char __user * ufilename, char *arguments, int retval);
// Add this back in..
// _char *audit_copy_exec_strings(char **argv);
int _audit_exit(int retval);
int _audit_fork(int retval);
int _audit_open(const char * kfile,const char __user *ufile, int flags, int mode, int retval);
int _audit_mkdir(const char * kfile, const char __user * ufile, int mode, int retval);
int _audit_unlink(const char * kfile,const char __user * ufile, int retval);
int _audit_rmdir(const char * kfile, const char __user * ufile, int retval);
int _audit_chown(const char * kfile,const char __user * ufile, uid_t user, gid_t group, int retval);
int _audit_chmod(const char * kfile, const char __user * ufile, mode_t mode, int retval);
int _audit_symlink(const char * kfrom,const char __user * ufrom, const char * kto,const char __user * uto, int retval);
int _audit_link(const char * kfrom,const char __user * ufrom, const char * kto,const char __user * uto, int retval);
int _audit_rename(const char * kfrom,const char __user * ufrom, const char * kto, const char __user * uto, int retval);
int _audit_truncate(const char * kfile,const char __user * ufile, loff_t length, int retval);
int _audit_ftruncate(const char * file, loff_t length, int retval);
int _audit_chroot(const char * kfile, const char * ufile, int retval);
int _audit_setuid(uid_t uid, int retval);
int _audit_setreuid(uid_t ruid, uid_t euid, int retval);
int _audit_setresuid(uid_t ruid, uid_t euid, uid_t suid, int retval);
int _audit_setgid(gid_t gid, int retval);
int _audit_setregid(gid_t rgid, gid_t egid, int retval);
int _audit_setresgid(gid_t rgid, gid_t egid, gid_t sgid, int retval);
int _audit_create_module(const char * kname, const char __user * uname, int retval);
int _audit_delete_module(const char * kname, const char __user * uname, int retval);
int _audit_reboot(int magic1, int magic2, unsigned int cmd, void __user * arg, int retval);
int _audit_connect(int sockfd, struct sockaddr __user *serv_addr, int addrlen, int retval);
int _audit_accept(int sockfd, struct sockaddr __user *serv_addr, int *addrlen, int retval);
int _audit_mount(const char *kdev_name, const char __user *udev_name, const char *kdir_name, const char __user *udir_name, unsigned long flags, int retval);
int _audit_umount(const char *kname, const char __user *uname, int flags, int retval);

void saudit_init(void);
char * saudit_copy_argv(char __user *__user *argv);

extern int AUDIT_IS_RUNNING;

// Inline routines in order to speed things up a little.
static inline int saudit_mknod(const char * kfile, const char * ufile, int mode, dev_t dev, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_mknod(kfile,ufile,mode,dev,retval));
	else
		return(0);
}
static inline int saudit_execve(const char * kfilename, const char * ufilename, char *arguments, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_execve(kfilename,ufilename,arguments,retval));
	else
		return(0);
}
static inline int saudit_exit(int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_exit(retval));
	else
		return(0);
}
static inline int saudit_fork(int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_fork(retval));
	else
		return(0);
}
static inline int saudit_open(const char * kfile,const char *ufile, int flags, int mode, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_open(kfile,ufile,flags,mode,retval));
	else
		return(0);
}
static inline int saudit_mkdir(const char * kfile, const char * ufile, int mode, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_mkdir(kfile,ufile,mode,retval));
	else
		return(0);
}
static inline int saudit_unlink(const char * kfile,const char * ufile, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_unlink(kfile,ufile,retval));
	else
		return(0);
}
static inline int saudit_rmdir(const char * kfile, const char * ufile, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_rmdir(kfile,ufile,retval));
	else
		return(0);
}
static inline int saudit_chown(const char * kfile,const char * ufile, uid_t user, gid_t group, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_chown(kfile,ufile,user,group,retval));
	else
		return(0);
}
static inline int saudit_chmod(const char * kfile, const char * ufile, mode_t mode, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_chmod(kfile,ufile,mode,retval));
	else
		return(0);
}
static inline int saudit_symlink(const char * kfrom,const char * ufrom, const char * kto,const char * uto, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_symlink(kfrom,ufrom,kto,uto,retval));
	else
		return(0);
}
static inline int saudit_link(const char * kfrom,const char * ufrom, const char * kto,const char * uto, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_link(kfrom,ufrom,kto,uto,retval));
	else
		return(0);
}
static inline int saudit_rename(const char * kfrom,const char * ufrom, const char * kto, const char * uto, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_rename(kfrom,ufrom,kto,uto,retval));
	else
		return(0);
}
static inline int saudit_truncate(const char * kfile,const char * ufile, loff_t length, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_truncate(kfile,ufile,length,retval));
	else
		return(0);
}
static inline int saudit_ftruncate(const char * file, loff_t length, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_ftruncate(file,length,retval));
	else
		return(0);
}
static inline int saudit_chroot(const char * kfile, const char * ufile, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_chroot(kfile,ufile,retval));
	else
		return(0);
}
static inline int saudit_setuid(uid_t uid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setuid(uid,retval));
	else
		return(0);
}
static inline int saudit_setreuid(uid_t ruid, uid_t euid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setreuid(ruid,euid,retval));
	else
		return(0);
}
static inline int saudit_setresuid(uid_t ruid, uid_t euid, uid_t suid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setresuid(ruid,euid,suid,retval));
	else
		return(0);
}
static inline int saudit_setgid(gid_t gid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setgid(gid,retval));
	else
		return(0);
}
static inline int saudit_setregid(gid_t rgid, gid_t egid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setregid(rgid,egid,retval));
	else
		return(0);
}
static inline int saudit_setresgid(gid_t rgid, gid_t egid, gid_t sgid, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_setresgid(rgid,egid,sgid,retval));
	else
		return(0);
}
static inline int saudit_create_module(const char * kname, const char * uname, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_create_module(kname,uname,retval));
	else
		return(0);
}
static inline int saudit_delete_module(const char * kname, const char * uname, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_delete_module(kname,uname,retval));
	else
		return(0);
}
static inline int saudit_reboot(int magic1, int magic2, unsigned int cmd, void __user * arg, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_reboot(magic1,magic2,cmd,arg,retval));
	else
		return(0);
}
static inline int saudit_connect(int sockfd, void __user *serv_addr, int addrlen, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_connect(sockfd,serv_addr,addrlen,retval));
	else
		return(0);
}
static inline int saudit_accept(int sockfd, void __user *serv_addr, int *addrlen, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_accept(sockfd,serv_addr,addrlen,retval));
	else
		return(0);
}
static inline int saudit_mount(const char *kdev_name, const char *udev_name, const char *kdir_name, const char *udir_name, unsigned long flags, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_mount(kdev_name,udev_name,kdir_name,udir_name,flags,retval));
	else
		return(0);
}
static inline int saudit_umount(const char *kname, const char *uname, int flags, int retval) {
	if (unlikely(AUDIT_IS_RUNNING))
		return (_audit_umount(kname,uname,flags,retval));
	else
		return(0);
}

#endif /* __KERNEL__ */

#else	/* CONFIG_C2_AUDIT */

#ifdef __KERNEL__
static inline int saudit_mknod(const char * kfile, const char * ufile, int mode, dev_t dev, int retval) {return(0);}
static inline char * saudit_copy_argv(char **argv) {return(0);}
static inline int saudit_execve(const char * kfilename, const char * ufilename, char *arguments, int retval) {return(0);}
static inline int saudit_exit(int retval) {return(0);}
static inline int saudit_fork(int retval) {return(0);}
static inline int saudit_open(const char * kfile,const char *ufile, int flags, int mode, int retval) {return(0);}
static inline int saudit_mkdir(const char * kfile, const char * ufile, int mode, int retval) {return(0);}
static inline int saudit_unlink(const char * kfile,const char * ufile, int retval) {return(0);}
static inline int saudit_rmdir(const char * kfile, const char * ufile, int retval) {return(0);}
static inline int saudit_chown(const char * kfile,const char * ufile, uid_t user, gid_t group, int retval) {return(0);}
static inline int saudit_chmod(const char * kfile, const char * ufile, mode_t mode, int retval) {return(0);}
static inline int saudit_symlink(const char * kfrom,const char * ufrom, const char * kto,const char * uto, int retval) {return(0);}
static inline int saudit_link(const char * kfrom,const char * ufrom, const char * kto,const char * uto, int retval) {return(0);}
static inline int saudit_rename(const char * kfrom,const char * ufrom, const char * kto, const char * uto, int retval) {return(0);}
static inline int saudit_truncate(const char * kfile,const char * ufile, loff_t length, int retval) {return(0);}
static inline int saudit_ftruncate(const char * file, loff_t length, int retval) {return(0);}
static inline int saudit_chroot(const char * kfile, const char * ufile, int retval) {return(0);}
static inline int saudit_setuid(uid_t uid, int retval) {return(0);}
static inline int saudit_setreuid(uid_t ruid, uid_t euid, int retval) {return(0);}
static inline int saudit_setresuid(uid_t ruid, uid_t euid, uid_t suid, int retval) {return(0);}
static inline int saudit_setgid(gid_t gid, int retval) {return(0);}
static inline int saudit_setregid(gid_t rgid, gid_t egid, int retval) {return(0);}
static inline int saudit_setresgid(gid_t rgid, gid_t egid, gid_t sgid, int retval) {return(0);}
static inline int saudit_create_module(const char * kname, const char * uname, int retval) {return(0);}
static inline int saudit_delete_module(const char * kname, const char * uname, int retval) {return(0);}
static inline int saudit_reboot(int magic1, int magic2, unsigned int cmd, void __user * arg, int retval) {return(0);}
static inline int saudit_connect(int sockfd, void __user *serv_addr, int addrlen, int retval) {return(0);}
static inline int saudit_accept(int sockfd, void __user *serv_addr, int *addrlen, int retval) {return(0);}
static inline int saudit_mount(const char *kdev_name, const char *udev_name, const char *kdir_name, const char *udir_name, unsigned long flags, int retval) {return(0);}
static inline int saudit_umount(const char *kname, const char *uname, int flags, int retval) {return(0);}
#endif /* __KERNEL__ */
#endif /* CONFIG_C2_AUDIT */
#endif /* __C2_AUDIT_H */
